# XOSD status displaying plugin for Tkabber.
# See: http://ldots.org/xosd-guide/
#      http://ignavus.net/software.html
# Rewritten by Konstantin Khomoutov <khomoutov at gmail dot com>
# Original author: Jan Hudec (+ fixes by various people)
# $Id: osd.tcl 51 2006-09-28 00:15:58Z kostix $



namespace eval osd {
	# Run only under X11 (an work around Tk 8.3 which lacks "tk windowingsystem"):

	set can_run false

	if {[catch {set can_run [expr [tk windowingsystem] == "x11"]}]} {
		set can_run [expr {$tcl_platform(platform) == "unix"}]
	}

	if {! $can_run} return

	unset can_run

	# State variables:
	
	variable options
	variable pipes
	variable lasttime [clock clicks -milliseconds]
	variable lastmsg ""
	array set statuses {}

    # Configuration options:

    set group {XOSD Status Display}

	custom::defgroup Plugins [::msgcat::mc "Plugins options."] -group Tkabber

	custom::defgroup $group \
		[::msgcat::mc "options for XOSD plugin."] \
		-group Plugins

	# XOSD's osd_cat config:

	custom::defvar options(font) "fixed" \
		[::msgcat::mc "Font to use for displayed messages."] \
		-group $group -type string

	custom::defvar options(delay) 5 \
		[::msgcat::mc "How many seconds do displayed messages stay on screen."] \
		-group $group -type integer

	custom::defvar options(cast_shadows) 0 \
		[::msgcat::mc "Whether displayed messages cast shadows."] \
		-group $group -type boolean

	custom::defvar options(shadow_offset) 1 \
		[::msgcat::mc "Offset for drawing shadows. 0 disables shadows whatsoever."] \
		-group $group -type integer

	custom::defvar options(color) "red" \
		[::msgcat::mc "Text color."] \
		-group $group -type string

	# Plugin config:

	custom::defvar options(throttle_time) 1000 \
		[::msgcat::mc "How many milliseconds should pass before the new message will be accepted for display.\
			Messages arriving earlier will be silently dropped.\
			Set to 0 to disable throttling."] \
		-group $group -type string

	custom::defvar options(ignoredups) 0 \
		[::msgcat::mc "Ignore ajacent duplicated messages."] \
		-group $group -type boolean

    unset group
}



proc osd::initialize {connid args} {
	variable pipes
	variable options

	debugmsg plugins "OSD: connid is $connid"

	set command osd_cat

	append command " --font=$options(font)"

	if {$options(delay) > 0} {
		append command " --delay=$options(delay)"
	} elseif {$options(delay) < 0} {
		error [::msgcat::mc "Delay must be a positive integer value greater than zero"]
	}

	if {$options(color) != ""} {
		append command " --color=$options(color)"
	}

	if {$options(cast_shadows) && $options(shadow_offset) > 0} {
		append command " --shadow=$options(shadow_offset)"
	}

	debugmsg plugins "OSD: will open: $command"
	set pipes($connid) [open "| $command" w]
	fconfigure $pipes($connid) -buffering line
}



proc osd::deinitialize {connid args} {
	variable pipes

	debugmsg plugins "OSD: connid is $connid"

	if {$connid == {}} {
		set ids [array names pipes]
	} else {
		set ids $connid
	}

	foreach id $ids {
		if {[catch {close $pipes($id)} err]} {
			debugmsg plugins "OSD: error closing osd_cat pipe: $err"
		}
	}
}



proc osd::write {text {connid 1}} {
	variable pipes

	if {[catch {puts $pipes($connid) $text} err]} {
		debugmsg plugins "Can't write to OSD: $err"
	}
}



proc osd::presence_notify {connid from type x args} {
    variable statuses

    if {[catch  { set nick [get_nick $connid $from chat] }]} {
        set nick "$from" 
    }
    
    if { "$nick"!="$from" } { 
        set thefrom "$nick ($from)" 
    } else { 
        set thefrom "$from" 
    }
    if { "$type"=="" } { set type "available" }

    set status ""
    set show ""
    foreach {attr val} $args {
	switch -- $attr {
	    -status   {set status $val}
	    -show     {set show   $val}
	}
    }

    if {"$status"!=""} { set status " ($status)" }
    if {"$show"!=""} { set type "$type/$show" }

    set newstatus "$thefrom: $type$status"
    
    if {[catch { set oldstatus $statuses($from) } ]} {
        set oldstatus "$newstatus"
    }

    if { "$newstatus"!="$oldstatus" } {
        write "$newstatus"
    }

    set statuses($from) "$newstatus"
}



proc osd::chat_message_notify {chatid from type body extras} {
    if {[chat::is_our_jid $chatid $from] || ![cequal $type chat]} {
	return
    }
    set nick [get_nick [chat::get_connid $chatid] $from $type]
    write "New message from $nick"
}



proc osd::get_nick {connid jid type} {
    if {[catch {chat::get_nick $connid $jid $type} nick]} {
	return [chat::get_nick $jid $type]
    } else {
	return $nick
    }
}



proc osd::display {msg} {
	variable options
	variable lasttime
	variable lastmsg

	if {$options(throttle_time) > 0} {
		set now [clock clicks -milliseconds]
		set diff [expr {$now - $lasttime}]
		set lasttime $now

		if {$diff > $options(throttle_time)} return
	}

	if {$options(ignoredups)} {
		if {![cequal $lastmsg $msg]} {
			set lastmsg $msg
		} else return
	}

	write "[tk appname]: $msg"
}



namespace eval osd {
	::hook::add postload_hook [list [namespace current]::initialize 1]
	::hook::add quit_hook [list [namespace current]::deinitialize 1]

	#::hook::add connected_hook [namespace current]::initialize
	#::hook::add disconnected_hook [namespace current]::deinitialize

	::hook::add set_status_hook [namespace current]::display

if {0} {
	foreach event $::osd {
		switch -- $event {
			presence { hook::add client_presence_hook ::osd::presence_notify 100 }
			chat_message { hook::add draw_message_hook ::osd::chat_message_notify 20 }
		default { debugmsg plugins "Unsupported notify type $event" }
		}
	}
}
}

# vim:noet:ts=4:sw=4
